@@ -17,7 +17,7 @@ require DynaLoader;
 @EXPORT_OK = qw(
 	TACPLUS_CLIENT
 );
-$VERSION = '0.24';
+$VERSION = '0.25';
 
 sub new
 {
@@ -50,6 +50,7 @@ if ($res<0) {
                                $h{'Key'},
                                $h{'Timeout'} ? $h{'Timeout'} : 15
                               );
+	$self->{'open'} = 1 if ($res >= 0);
         last if ($res >= 0);
     }
 }
@@ -87,9 +88,21 @@ sub authen
 
 sub close
 {
-    deinit_tac_session();
+    my ($self) = @_;
+
+    if ($self->{'open'})
+    {
+	deinit_tac_session();
+	$self->{'open'} = 0;
+    }
 }
 
+sub DESTROY
+{
+    my ($self) = @_;
+
+    $self->close();
+}
 
 
 sub AUTOLOAD {
